VORTEX_MSG_TO_CHILD_FD = 0
VORTEX_MSG_FROM_CHILD_FD = 1
LOGGING_FROM_CHILD_FD = 2
VORTEX_UUID_TO_CHILD_FD = 3
VORTEX_UUID_FROM_CHILD_FD = 4
PLUGIN_STATE_TO_CHILD_FD = 5
PLUGIN_STATE_FROM_CHILD_FD = 6
